// Generated by CoffeeScript 1.3.3
var clients, fs, options, packAssets, path, systemAssets;

require('colors');

fs = require('fs');

path = require('path');

systemAssets = require('./system');

packAssets = process.env['SS_PACK'];

options = {
  packedAssets: packAssets || false,
  liveReload: ['code', 'css', 'static', 'templates', 'views'],
  dirs: {
    code: '/client/code',
    css: '/client/css',
    "static": '/client/static',
    assets: '/client/static/assets',
    templates: '/client/templates',
    views: '/client/views',
    workers: '/client/workers'
  }
};

clients = {};

module.exports = function(ss, router) {
  var determineLatestId, formatters, http, templateEngine;
  templateEngine = require('./template_engine')(ss);
  formatters = require('./formatters')(ss);
  http = require('./http')(ss, clients, options);
  formatters.add('javascript');
  formatters.add('css');
  formatters.add('html');
  determineLatestId = function(client) {
    var files, id, latestId;
    try {
      files = fs.readdirSync(path.join(ss.root, options.dirs.assets, client.name));
      latestId = files.sort().pop();
      id = latestId.split('.')[0];
      if (id.length !== 13) {
        throw 'Invalid Client ID length';
      }
      return id;
    } catch (e) {
      return false;
    }
  };
  systemAssets.load();
  return {
    formatters: formatters,
    templateEngine: templateEngine,
    assets: systemAssets,
    options: options,
    set: function(newOption) {
      var k, v, x, y, _results;
      if (typeof newOption !== 'object') {
        throw new Error('ss.client.set() takes an object e.g. {liveReload: false}');
      }
      _results = [];
      for (k in newOption) {
        v = newOption[k];
        if (v instanceof Object) {
          _results.push((function() {
            var _results1;
            _results1 = [];
            for (x in v) {
              y = v[x];
              _results1.push(options[k][x] = y);
            }
            return _results1;
          })());
        } else {
          _results.push(options[k] = v);
        }
      }
      return _results;
    },
    packAssets: function(opts) {
      if (opts && typeof opts !== 'object') {
        throw new Error('Options passed to ss.client.packAssets() must be an object');
      }
      options.packedAssets = opts || true;
      if (options.packedAssets && process.listeners('uncaughtException').length === 0) {
        return process.on('uncaughtException', function(err) {
          console.log('Uncaught Exception!'.red);
          return console.error(err.stack);
        });
      }
    },
    define: function(name, paths) {
      var _this = this;
      if (clients[name] != null) {
        throw new Error("Client name '" + name + "' has already been defined");
      }
      if (typeof paths.view !== 'string') {
        throw new Error("You may only define one HTML view per single-page client. Please pass a filename as a string, not an Array");
      }
      if (paths.view.indexOf('.') === -1) {
        throw new Error("The '" + paths.view + "' view must have a valid HTML extension (such as .html or .jade)");
      }
      if (paths.templates) {
        paths.tmpl = paths.templates;
      }
      ['css', 'code', 'tmpl'].forEach(function(assetType) {
        if (!(paths[assetType] instanceof Array)) {
          return paths[assetType] = [paths[assetType]];
        }
      });
      return clients[name] = {
        id: Number(Date.now()),
        name: name,
        paths: paths
      };
    },
    load: function() {
      var client, id, name, pack;
      ss.client.formatters = formatters.load();
      ss.client.templateEngines = templateEngine.load();
      systemAssets.send('code', 'init', "require('/entry');");
      if (options.packedAssets) {
        if (!packAssets) {
          ss.log('i'.green, "Attempting to find pre-packed assets... (force repack with SS_PACK=1)".grey);
          for (name in clients) {
            client = clients[name];
            if (id = options.packedAssets.id || determineLatestId(client)) {
              client.id = id;
              ss.log('✓'.green, ("Serving client '" + client.name + "' using pre-packed assets (ID " + client.id + ")").grey);
            } else {
              ss.log('!'.red, ("Unable to find pre-packed assets for '" + client.name + "'. All assets will be repacked").grey);
              packAssets = true;
            }
          }
        }
        if (packAssets) {
          pack = require('./pack');
          for (name in clients) {
            client = clients[name];
            pack(ss, client, options);
          }
        }
      } else {
        require('./serve/dev')(ss, router, options);
        if (options.liveReload) {
          require('./live_reload')(ss, options);
        }
      }
      return require('./serve/ondemand')(ss, router, options);
    }
  };
};
